from bkmrks import bkmrks, urls


def generate(md_file_name="public/index", catalog="index"):
    bookmarks = bkmrks.get(catalog=catalog)
    md_file_content = ""
    if len(bookmarks) > 0:
        for line in bookmarks.values():
            if len(line) > 0:
                for item in line.values():
                    md_file_content += md_a_img(item)
            md_file_content += md_hr()
    if len(md_file_content) < 10:
        return None

    md_file_name = md_file_name.split(".")[0] + ".md"
    with open(md_file_name, "+w") as f:
        f.write(md_file_content)
    return md_file_name


def md_a_img(item):
    try:
        url = item["url"]
    except:
        return ""
    if "name" in item:
        name = item["name"]
    else:
        name = urls.get_name_by_url(url=url)

    if "img" in item:
        img = item["img"]
    else:
        img = urls.get_default_img(text=name)

    return f'\n[![{name}]({img})]({url} "{name}")'


def md_hr():
    return "\n---"



import argparse
import os

from bkmrks import bkmrks, presenter


def main():
    parser = argparse.ArgumentParser(prog="bkmrks")
    parser.add_argument("--version", action="version", version="%(prog)s v0.2.2")

    subparsers = parser.add_subparsers(dest="command", help="Available commands")
    render_parser = subparsers.add_parser(
        "render",
        help="✍️   Render your html bookmarks to public folder.",
    )

    load_parser = subparsers.add_parser(
        "load",
        help="📚 Load and HTML file/url and create a bookmark page.",
    )

    load_parser.add_argument(
        "-html",
        help="HTML file/url to scrape.",
    )

    load_parser.add_argument(
        "-catalog",
        help="Bookmark catalog name.",
        default="index",
    )

    add_parser = subparsers.add_parser(
        "add",
        help="✍️   Add a bookmark to catalog.",
    )

    add_parser.add_argument(
        "url",
        help="URL bookrmark to add to the catalog.",
    )

    add_parser.add_argument(
        "-catalog",
        help="Bookmark catalog name.",
        default="index",
    )

    add_parser.add_argument(
        "-l",
        help="Line of the catalog to add your bookmark.",
        default="1",
    )

    add_parser.add_argument(
        "-pos",
        help="Position in the line of the catalog to add your bookmark.",
        default="0",
    )

    rm_parser = subparsers.add_parser(
        "rm",
        help="❌ Remove a bookmark from catalog.",
    )

    rm_parser.add_argument(
        "-catalog",
        help="Bookmark catalog name.",
        default="index",
    )

    rm_parser.add_argument(
        "-l",
        help="Line of the catalog to remove your bookmark.",
        default="1",
    )

    rm_parser.add_argument(
        "-pos",
        help="Position in the line of the catalog to remove your bookmark.",
        default="1",
    )

    mv_parser = subparsers.add_parser(
        "mv",
        help="🔄 Move a bookmark from a catalog to another.",
    )

    mv_parser.add_argument(
        "-f_catalog",
        help="Bookmark catalog name to move from.",
        default="index",
    )

    mv_parser.add_argument(
        "-f_l",
        help="Line of the catalog to move from.",
        default="1",
    )

    mv_parser.add_argument(
        "-f_pos",
        help="Position in the line of the catalog to move from.",
        default="1",
    )

    mv_parser.add_argument(
        "-t_catalog",
        help="Bookmark catalog name to move to.",
        default="index",
    )

    mv_parser.add_argument(
        "-t_l",
        help="Line of the catalog to move to.",
        default="1",
    )

    mv_parser.add_argument(
        "-t_pos",
        help="Position in the line of the catalog to move to.",
        default="1",
    )

    args = parser.parse_args()
    if args.command == "render":
        presenter.render()

    if args.command == "load":
        bkmrks.html2catalog(html_file_name=args.html, catalog=args.catalog)
        presenter.render()

    if args.command == "add":
        bkmrks.add_url(
            url=str(args.url),
            catalog=str(args.catalog),
            line_index=int(args.l),
            item_index=int(args.pos),
        )
        presenter.render()

    if args.command == "rm":
        bkmrks.remove_url(
            catalog=str(args.catalog), line_index=int(args.l), item_index=int(args.pos)
        )
        presenter.render()

    if args.command == "mv":
        bkmrks.mv_url(
            from_catalog=args.f_catalog,
            from_line_index=args.f_l,
            from_item_index=args.f_pos,
            to_catalog=args.to_catalog,
            to_line_index=args.to_l,
            to_item_index=args.to_pos,
        )
        presenter.render()
    return


if __name__ == "__main__":
    main()



import os

import yaml
from bs4 import BeautifulSoup

from bkmrks import urls


def catalogs_folder():
    return "catalogs"


def get(catalog="index"):
    ensure_catalogs_folder()
    catalog = catalog.split(".")[0] + ".yaml"

    if not os.path.exists(f"{catalogs_folder()}/{catalog}"):
        return {}

    with open(f"{catalogs_folder()}/{catalog}", "r") as f:
        catalog_data = yaml.safe_load(f.read())
        if catalog_data is None:
            return {}
        else:
            return catalog_data


def set(data, catalog="index"):
    ensure_catalogs_folder()
    catalog = catalog.split(".")[0] + ".yaml"
    with open(f"{catalogs_folder()}/{catalog}", "+w") as f:
        yaml.dump(data, f)
        f.seek(0)
    return get(catalog=catalog)


def html2catalog(html_file_name, catalog):
    domain = urls.extract_domain_from_url(url=html_file_name)
    html = urls.read_from_url_or_path(url_path=html_file_name)

    soup = BeautifulSoup(html, features="html.parser")
    soup_all_hr_a_tags = soup.find_all(["hr", "a"])

    line_index = 1
    item_index = 1
    catalog_data = {}
    line_name = get_line_name(line_index=line_index)
    catalog_data[line_name] = {}

    for soup_item in soup_all_hr_a_tags:
        if soup_item.has_attr("href") and not soup_item["href"].startswith("#"):

            img = urls.get_img_from_a_soup_item(soup_item=soup_item, domain=domain)
            url = urls.ensure_domain(url=soup_item["href"], domain=domain)
            name = urls.get_name_by_url(url=url)

            bookmark_item = get_bookmark_item(url=url, name=name, img=img)

            item_name = get_item_name(item_index=item_index)
            catalog_data[line_name][item_name] = bookmark_item.copy()

            item_index += 1

    set(data=catalog_data, catalog=catalog)


def mv_url(
    from_catalog="index",
    from_line_index=1,
    from_item_index=0,
    to_catalog="index",
    to_line_index=1,
    to_item_index=0,
):
    url = get_url(
        catalog=from_catalog,
        line_index=from_line_index,
        item_index=from_item_index,
    )
    if url is None:
        return
    add_url(
        url=url,
        catalog=to_catalog,
        line_index=to_line_index,
        item_index=to_item_index,
    )
    remove_url(
        catalog=from_catalog,
        line_index=from_line_index,
        item_index=from_item_index,
    )
    return True


def add_url(url, catalog="index", line_index=1, item_index=0):
    return edit_bookmark(
        url=url,
        catalog=catalog,
        line_index=line_index,
        item_index=item_index,
        action="add",
    )


def remove_url(catalog="index", line_index=1, item_index=0):
    return edit_bookmark(
        url="",
        catalog=catalog,
        line_index=line_index,
        item_index=item_index,
        action="rm",
    )


def edit_bookmark(url, catalog="index", line_index=1, item_index=0, action="add"):
    catalog_data = get(catalog=catalog)
    catalog_data_new = {}
    line_name = None
    item_name = None
    if action == "rm" and catalog_data == {}:
        return
    if len(catalog_data) < line_index and action == "add":
        catalog_data_new = catalog_data.copy()

        line_name = get_line_name(line_index=len(catalog_data) + 1)
        item_name = get_item_name(item_index=1)

        catalog_data_new[line_name] = {}
        catalog_data_new[line_name][item_name] = {}
    else:
        line_index = at_least_1(line_index)
        item_index = at_least_1(item_index)
        i = 0
        for catalog_line_name, catalog_line in catalog_data.items():
            i += 1
            if len(catalog_line_name) < 8:
                catalog_line_name = get_line_name(line_index=i)
            if line_index == i:
                line_name = catalog_line_name
                catalog_data_new[catalog_line_name] = {}
                j = 0
                for catalog_line_item in catalog_line.values():
                    j += 1
                    if item_index == j and action == "add":
                        catalog_data_new[catalog_line_name][
                            get_item_name(item_index=j)
                        ] = {}
                        item_name = get_item_name(item_index=j)
                        j += 1
                    if item_index == j and action == "rm":
                        print("")
                    else:
                        catalog_data_new[catalog_line_name][
                            get_item_name(item_index=j)
                        ] = catalog_line_item.copy()
                if item_name is None and action == "add":
                    j += 1
                    catalog_data_new[catalog_line_name][get_item_name(item_index=j)] = (
                        {}
                    )
                    item_name = get_item_name(item_index=j)
            else:
                catalog_data_new[catalog_line_name] = catalog_line.copy()
    if action != "rm":
        catalog_data_new[line_name][item_name] = parse_url(url=url)

    set(data=catalog_data_new, catalog=catalog)
    return True


def get_url(
    catalog="index",
    line_index=1,
    item_index=1,
):
    url = None
    line_index = at_least_1(line_index)
    item_index = at_least_1(item_index)

    catalog_data = get(catalog=catalog)
    if len(catalog_data) == 0:
        return
    if len(list(catalog_data.values())) >= line_index:
        catalog_line = list(catalog_data.values())[line_index - 1]
        if len(list(catalog_line.values())) >= item_index:
            if "url" in list(catalog_line.values())[item_index - 1]:
                url = list(catalog_line.values())[item_index - 1]["url"]

    return url


def get_line_name(line_index):
    line_name = f"line{line_index:04d}"
    return line_name


def get_item_name(item_index):
    item_name = f"item{item_index:04d}"
    return item_name


def parse_url(url, domain=None):
    if domain is not None:
        url = urls.ensure_domain(url=url, domain=domain)

    name = urls.get_name_by_url(url=url)

    bookmark_item = get_bookmark_item(
        url=url, name=name, img=urls.get_url_icon(url=url)
    )
    return bookmark_item


def get_bookmark_item(url, name, img):
    bookmark_item = {}
    bookmark_item["name"] = name
    bookmark_item["url"] = url
    bookmark_item["img"] = img

    return bookmark_item


def ensure_catalogs_folder():
    if not os.path.exists(catalogs_folder()):
        os.mkdir(catalogs_folder())
        data = {
            get_line_name(line_index=1): {
                get_item_name(item_index=1): get_bookmark_item(
                    url="https://github.com/bouli/bkmrks",
                    name="bkmrks_sample_page",
                    img="https://cesarcardoso.cc/README/1_bouli.png",
                )
            }
        }
        set(data=data)


def at_least_1(number):
    number = int(number)
    if number < 1:
        number = 1

    return number



import os

import markdown

from bkmrks import bkmrks, md


def render():
    bkmrks.ensure_catalogs_folder()
    ensure_public_folder()

    bookmarks = os.listdir(bkmrks.catalogs_folder())
    menu = []
    htmls = []
    for catalog in bookmarks:
        md_file_name = "public/" + catalog
        md_generated_file = md.generate(md_file_name=md_file_name, catalog=catalog)
        if md_generated_file is not None:
            htmls.append(generate_html(md_file_name))
            menu.append(
                get_file_and_set_variable(
                    file="templates/menu_item.html",
                    variable="menu_item",
                    content=catalog.split(".")[0],
                )
            )
    menu = " | ".join(menu)
    for html in htmls:
        html_content = get_file_and_set_variable(
            file=html,
            variable="menu",
            content=menu,
        )
        with open(html, "+w") as f:
            f.write(html_content)


def generate_html(md_file="public/index", template="index"):
    ensure_public_folder()
    md_file = md_file.split(".")[0] + ".md"
    with open(md_file, "r") as fm:
        html = set_template_content(
            markdown.markdown(fm.read()), template, extension="html"
        )
        html_file = md_file.split(".")[0] + ".html"
        with open(html_file, "+w") as fh:
            fh.write(html)
    return html_file


def get_file_and_set_variable(file, variable, content):
    with open(file, "r") as f:
        file_content = f.read().replace("{" + variable + "}", content)
    return file_content


def get_template(base_file_name, extension="html"):
    ensure_template_folder()
    template = "{" + extension + "}"

    html_file = "templates/" + base_file_name.split(".")[0] + f".{extension}"
    if os.path.exists(html_file):
        with open(html_file, "r") as f:
            template = f.read()

        if extension == "html":
            dirs = os.listdir("templates")
            for file in dirs:
                if (
                    file.split(".")[1] != "html"
                    and file.split(".")[0] == base_file_name.split(".")[0]
                ):
                    innerextension = file.split(".")[1]
                    innerfile = "templates/" + file
                    with open(innerfile, "r") as f:
                        innertemplate = f.read()
                        template = template.replace(
                            "{" + innerextension + "}", innertemplate
                        )

    return template


def set_template_content(content, base_file_name, extension="html"):
    ensure_template_folder()
    template = get_template(base_file_name, extension=extension)
    return template.replace("{" + extension + "}", content)


def ensure_public_folder():
    if not os.path.exists("public"):
        os.mkdir("public")


def ensure_template_folder():
    if not os.path.exists("templates"):
        os.mkdir("templates")
        default_templates_dir = os.path.join(
            os.path.dirname(os.path.realpath(__file__)), "templates"
        )
        files = os.listdir(default_templates_dir)
        for file in files:
            if file.split(".")[1] in ["css", "html"]:
                with open(os.path.join(default_templates_dir, file), "r+") as fr:
                    with open(os.path.join("templates", file), "+w") as fw:
                        fw.write(fr.read())



from urllib.parse import urlparse

import requests
from bs4 import BeautifulSoup


def ensure_domain(url, domain):
    if not url.startswith("https"):
        if url.startswith("/"):
            url = domain + url
        else:
            url = domain + "/" + url
    return url


def get_url_icon(url):
    href_parse = urlparse(url)
    domain = "://".join([href_parse.scheme, href_parse.netloc])
    img = domain + "/favicon.ico"
    try:
        r = requests.get(url)
    except:
        return img

    soup = BeautifulSoup(r.text, features="html.parser")
    soup_link = soup.find_all(
        "link",
        attrs={
            "rel": [
                "icon",
                "apple-touch-icon",
            ]
        },
    )
    if len(soup_link) > 0:
        icon = None
        for icon_item in soup_link:
            if not icon_item.has_attr("sizes"):
                icon_item["sizes"] = "16x16"
            if icon is None:
                icon = icon_item
            else:
                if len(icon_item["sizes"]) < 9:

                    if (
                        len(icon_item["sizes"]) > len(icon_item["sizes"])
                        or icon_item["sizes"].split("x")[0]
                        > icon["sizes"].split("x")[0]
                    ):
                        icon = icon_item
        img = icon["href"]

    img = ensure_domain(url=img, domain=domain)
    if img.split("/")[-1] == "favicon.ico":
        r = requests.get(img)
        if r.status_code == 404:
            if soup.find("img") is None:
                r = requests.get(domain)
                if r.status_code != 404:
                    soup = BeautifulSoup(r.text, features="html.parser")
                    if soup.find("img") is not None:
                        img = soup.find("img")["src"]
                    else:
                        img = get_default_img(text=href_parse.netloc)
            else:
                img = soup.find("img")["src"]
    if not img.startswith("http"):
        if img.startswith("/"):
            separator = ""
        else:
            separator = "/"
        img = separator.join([domain, img])
    return img


def get_default_img(text):
    return f"https://ui-avatars.com/api/?name={text}"


def get_name_by_url(url):
    href_parse = urlparse(url)
    name = href_parse.netloc.split(".")

    if name[-2] == "google":
        name = "_".join(name[:-1][::-1])
    else:
        name = name[-2]
    return name


def extract_domain_from_url(url):
    domain = ""

    url_parse = urlparse(url)
    if len(url_parse.netloc) > 0:
        domain = "://".join([url_parse.scheme, url_parse.netloc])
    return domain


def read_from_url_or_path(url_path):
    if str(url_path).startswith("https://"):
        content = requests.get(url_path).text

    else:
        url_path = url_path.split(".")[0] + ".html"
        with open(url_path, "r") as f:
            content = f.read()
    return content


def get_img_from_a_soup_item(soup_item, domain):
    soup_item["href"] = ensure_domain(url=soup_item["href"], domain=domain)

    use_soup_img = False
    if len(soup_item.find_all("img")) > 0:
        if len(soup_item.find("img")["src"]) < 200:
            use_soup_img = False
        else:
            use_soup_img = True

    if use_soup_img:
        img = soup_item.find("img")["src"]
    else:
        img = get_url_icon(soup_item["href"])
    return img
